/****************************************************************************

 MESSAGE PASSING INTERFACE TEST CASE SUITE

 Copyright IBM Corp. 1995

 IBM Corp. hereby grants a non-exclusive license to use, copy, modify, and
 distribute this software for any purpose and without fee provided that the
 above copyright notice and the following paragraphs appear in all copies.

 IBM Corp. makes no representation that the test cases comprising this
 suite are correct or are an accurate representation of any standard.

 In no event shall IBM be liable to any party for direct, indirect, special
 incidental, or consequential damage arising out of the use of this software
 even if IBM Corp. has been advised of the possibility of such damage.

 IBM CORP. SPECIFICALLY DISCLAIMS ANY WARRANTIES INCLUDING, BUT NOT LIMITED
 TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS AND IBM
 CORP. HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
 ENHANCEMENTS, OR MODIFICATIONS.

****************************************************************************

 These test cases reflect an interpretation of the MPI Standard.  They are
 are, in most cases, unit tests of specific MPI behaviors.  If a user of any
 test case from this set believes that the MPI Standard requires behavior
 different than that implied by the test case we would appreciate feedback.

 Comments may be sent to:
    Richard Treumann
    treumann@kgn.ibm.com

****************************************************************************
*/
//#include "mpi.h"
import mpi.*;

class attr {
  static public void main(String[] args) {
    
    //int class,rc,me,tasks,i,*val,*keyval,flag,key,temp, valdata;
    int rc,me,tasks,i,val,key,temp, valdata;
    boolean flag;
    //MPI_Comm comm;
    
    //#define TAG_UB (1<<30)-1 
    final int TAG_UB = (1<<30)-1;
    
    
    //MPI_Init(&argc,&argv);
    MPI.Init(args);
    //MPI_Comm_rank(MPI_COMM_WORLD,&me);
    me = MPI.COMM_WORLD.Rank();
    //MPI_Comm_size(MPI_COMM_WORLD,&tasks); 
    tasks = MPI.COMM_WORLD.Size();



    //MPI_Attr_get(MPI_COMM_WORLD,MPI_TAG_UB,&val,&flag);
    //Integer val = (Integer) MPI.COMM_WORLD.Attr_get(MPI.TAG_UB);
    val = MPI.COMM_WORLD.Attr_get(MPI.TAG_UB);
    //if(!flag) printf("ERROR in MPI.Attr_get: no val for MPI_TAG_UB\n");
    //if(*val != TAG_UB) {
    if(val != TAG_UB) {
      System.out.println
	("WARNING in MPI.Attr_get: tag_ub = "+val+", expected be "+TAG_UB);
      System.out.println("This is an error only if tag_ub < 32767");
    }


    //MPI_Attr_get(MPI_COMM_WORLD,MPI_HOST,&val,&flag);
    val = MPI.COMM_WORLD.Attr_get(MPI.HOST);
    //if(!flag) printf("ERROR in MPI_Attr_get: no val for MPI_HOST\n");
    //if(*val != MPI_PROC_NULL)
    if(val != MPI.PROC_NULL)
      System.out.println
	("ERROR in MPI_Attr_get: host = "+val+", should be "+MPI.PROC_NULL);



    //MPI_Attr_get(MPI_COMM_WORLD,MPI_IO,&val,&flag);
    val = MPI.COMM_WORLD.Attr_get(MPI.IO);
    //if(!flag) printf("ERROR in MPI.Attr_get: no val for MPI_IO\n");
    //if(*val != MPI_ANY_SOURCE)
    if(val != MPI.ANY_SOURCE)
      System.out.println
	("ERROR in MPI.Attr_get: io = "+val+", should be "+MPI.ANY_SOURCE);



    /****************  NOT SUPPORT FOLLOWING FUNCTION YET !!!

    //MPI_Keyval_create(MPI_NULL_COPY_FN,MPI_NULL_DELETE_FN,&key,0);
    key = MPI.Keyval_create(MPI.CALL_BACK_NULL) ;


    //MPI_Comm_dup(MPI_COMM_WORLD,&comm);
    Comm comm = MPI.COMM_WORLD.Dup();

    //MPI_Attr_get(comm,MPI_TAG_UB,&val,&flag);
    val = (Integer) comm.Attr_get(MPI.TAG_UB);
    //if(!flag) {
    if(val != null) {
      System.out.println
	("WARNING in MPI_Attr_get: no val for MPI.TAG_UB on dup'ed comm");
      System.out.println
	("This is not an error; the standard does not require that these attributes be copied" );
      //val = &valdata;
    }
    else {
      if(val.intValue() != TAG_UB)
	System.out.println
	  ("ERROR in MPI_Attr_get: tag_ub = "+val+", should be "+TAG_UB);
    }


    val = new Integer(12345);
    //MPI_Attr_put(comm,key,val);
    comm.Attr_put(key,val);
    //MPI_Attr_get(comm,key,&keyval,&flag);
    Integer keyval = (Integer) comm.Attr_get(key);
    //if(flag == 0)
    if(val != null) 
      System.out.println("ERROR in MPI.Attr_get: flag is false");
    if(keyval.intValue() != 12345)
      System.out.println
	("ERROR in MPI_Attr_get: val = "+keyval+", should be 12345");
    
    //MPI_Errhandler_set(comm,MPI_ERRORS_RETURN);

    temp = key;
    //MPI_Keyval_free(&key);
    MPI.Keyval_free(key);
    //if(key != MPI_KEYVAL_INVALID)
    //System.out.println("ERROR in MPI_Keyval_free: key not set to INVALID\n");

    // Note that this is erroneous use of a keyval; the standard 
    //  does not specify any particular behavior 
    key = temp;
    //rc = MPI_Attr_get(comm,key,&keyval,&flag);
    keyval = (Integer) comm.Attr_get(key);
    //MPI_Error_class(rc,&class);
    //if(class != MPI_ERR_OTHER) {
    //if (rc != MPI_SUCCESS) {
    //System.out.println("WARNING in MPI_Keyval_free: key not freed\n");
    //System.out.println("error returned was %d(%d)\n", rc, class );
    //}
    //else {
    //System.out.println("WARNING in MPI_Keyval_free: key not freed\n");
    //System.out.println("This is not an error but a failure to detect a user-error\n");
    //}
    //}

    //rc = MPI_Attr_delete(comm,MPI_TAG_UB);
    comm.Attr_delete(MPI.TAG_UB);
    //if(rc == MPI_SUCCESS)
    //System.out.println("ERROR in MPI_Attr_delete, no error detected\n"); 

    //rc = MPI_Attr_delete(comm,MPI_HOST);
    comm.Attr_delete(MPI.HOST);
    //if(rc == MPI_SUCCESS)
    //System.out.println("ERROR in MPI_Attr_delete, no error detected\n"); 

    //rc = MPI_Attr_delete(comm,MPI_IO);
    comm.Attr_delete(MPI.IO);
    //if(rc == MPI_SUCCESS)
    //System.out.println("ERROR in MPI_Attr_delete, no error detected\n"); 


    //MPI_Barrier(comm);
    Intracomm intcomm = (Intracomm) comm;
    intcomm.Barrier();
*****************/

    //MPI_Comm_free( &comm );
    if(me == 0)  System.out.println("Attr TEST COMPLETE");
    MPI.Finalize();
  }
}
